/*

DESCRIPTION:
    Main source file for OAT module.

SYNTAX:  n/a

KNOWN BUGS:  n/a

COPYRIGHT:  (c) 2018, Brian Cornell, all rights reserved.

DEPENDENCIES:
    config.h - PIC configuration registers
    main.h - configuration constants for main.c

VERSION:  2.0, production release

MODIFICATION HISTORY:

    26 Jan 18, BWC, development release 1.0

    8 Oct 18, BWC, release 2.0.  Changed:
        + S_TM_INT delay increased to 5 seconds
        + S_TM_SETTLE delay decreased to 50mS
        + Moved display update code to separate function writeDsp
        + Reduced power-up time to temp sensor & unit select to extend battery life

        Added:
        + Version display when first powered up
        + Low battery check < 2.048V that displays 'Lb'
    

*/

//Config PIC configuration registers first
#include "config.h"

//System includes second
#include <pic.h>
#include <xc.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>

//User includes last
#include <binary.h>
#include "main.h"


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *                              Data objects
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
const unsigned char ChrMap[] = {LCD_CHR_0,LCD_CHR_1,LCD_CHR_2,LCD_CHR_3,LCD_CHR_4,\
LCD_CHR_5,LCD_CHR_6,LCD_CHR_7,LCD_CHR_8,LCD_CHR_9,LCD_CHR_BLK,LCD_CHR_B,LCD_CHR_L};



/*Function main.
 *
 * Returns:  none
 *
 * Parameters:  none
 *
*/
void main(void)
{
    int result;
    signed char dsp;
    unsigned char ones, tens, hdrd, neg, count, poll;

    //Configure system and pause for settle time
    Initialize();

    //Display version
    ones = VERMINOR;
    tens = VERMAJOR;
    hdrd = LCD_MAP_BLK;
    neg = LCD_CHR_POS;
    writeDsp(neg, hdrd, tens, ones);            //Update display
    __delay_ms(S_TM_INT);                       //Wait long enough for user to notice

    SWDTEN = TRUE;                              //Enable watchdog timer

    //Infinite loop to get & display OAT.
    while (TRUE)
    {
        //Start / wake up from sleep...
        ADCON0bits.ON = TRUE;                   //Enable ADC module
        ADPCH = ADC_CH_FVR2;                    //Check battery voltage first
        __delay_ms(S_TM_SETTLE);                //Settle time

        for (poll = 0; (poll < 2); ++poll)
        {
            //Take multiple samples to average
            for (count = 0, result = 0; (count < S_QY_TMP_SAMPLES); ++count)
            {
                __delay_ms(S_TM_ACQ_INTERVAL);  //Acquisition time before next
                ADCON0bits.GO = TRUE;           //Start conversion
                while (ADCON0bits.GO);          //Wait for completion
                if (count > 0) result += ADRES; //Discard first conversion
            }

            result /= (S_QY_TMP_SAMPLES - 1);   //Calculate average

            if (!(poll))
            {
                if (result > S_LM_BAT_LIMIT) break; //Low battery; otherwise...
                PIN_O_TMPWR = TRUE;             //...power up temp sensor,
                S_EN_UNITS = TRUE;              //power up unit select pin,
                ADPCH = CFG_ADC_PCH;            //connect to temp sensor
            }
        }

        //Save energy
        ADCON0bits.ON = FALSE;                  //Disable ADC module
        PIN_O_TMPWR = FALSE;                    //Power down temp sensor

        //Passed low battery test, process temp reading
        if (poll)
        {
            //Calculate degrees C:  Ta=result/Tcoeff-T0c_offset
            dsp = (signed char) ((result / S_QY_TMP_COEFF) - S_QY_TMP_0COFFS);

            //Convert to F if selected
            if (PIN_I_UNITS) dsp = (((dsp * S_QY_F_CONVN) / S_QY_F_CONVD) + S_QY_F_OFFS);
            S_EN_UNITS = FALSE;                 //Power down unit select pin

            //Set / clear negative flag & convert dsp to + value if negative
            if (dsp < 0)
            {
                neg = LCD_CHR_NEG;
                dsp = ((dsp ^ 0xFF) + 1);       //Take absolute value
            }
            else neg = LCD_CHR_POS;

            //Separate dsp into individual values for 100, 10, 1 places
            if (!(hdrd = dsp / 100)) hdrd = LCD_MAP_BLK;
            else dsp -= 100;
            if ((!(tens = dsp / 10)) && (hdrd == LCD_MAP_BLK)) tens = LCD_MAP_BLK;
            ones = dsp % 10;
        }
        else
        {
            //Display low battery
            ones = LCD_MAP_B;
            tens = LCD_MAP_L;
            hdrd = LCD_MAP_BLK;
            neg = LCD_CHR_POS;
        }

        writeDsp(neg, hdrd, tens, ones);        //Update display

        SLEEP();                                //Sleep until watchdog wakes up

    }
} //End function main


/*Function writeDsp.  Writes digits to LCD display.
 *
 * Returns:  none
 *
 * Parameters:  sign = +/- indicator, either LCD_CHR_POS or LCD_CHR_NEG
 *              pos1 = LCD char 1 position (hundreds) offset into ChrMap
 *              pos2 = LCD char 2 position (tens) offset into ChrMap
 *              pos3 = LCD char 2 position (ones) offset into ChrMap
 *
*/
void writeDsp(unsigned char sign, unsigned char pos1, unsigned char pos2,\
        unsigned char pos3)
{
    //Wait until LCD module is ready
    while (!(LCDPSbits.WA));

    LCDDATA0 = ChrMap[pos3];
    LCDDATA1 = ChrMap[pos2];
    LCDDATA2 = (sign | (ChrMap[pos1] << 1));    //Merge sign & 100's place

    return;
} //End function writeDsp


/*Function Initialize.  Performs baseline MPU initialization at boot.
 *
 * Returns:  none
 *
 * Parameters:  none
 *
*/
void Initialize(void)
{
    di();                               //No interrupts

    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * Set port configuration.  To prevent any aberrant behaviors before 
     * initialization is complete set output latches.
     *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    //Port A
    PORTA = VL_PORTA;
    TRISA = IO_PORTA;
    ANSELA = AD_PORTA;
    WPUA = WP_PORTA;
    ODCONA = OD_PORTA;
    SLRCONA = SR_PORTA;
    INLVLA = IL_PORTA;
    //Port B
    PORTB = VL_PORTB;
    TRISB = IO_PORTB;
    ANSELB = AD_PORTB;
    WPUB = WP_PORTB;
    ODCONB = OD_PORTB;
    SLRCONB = SR_PORTB;
    INLVLB = IL_PORTB;
    //Port C
    PORTC = VL_PORTC;
    TRISC = IO_PORTC;
    ANSELC = AD_PORTC;
    WPUC = WP_PORTC;
    ODCONC = OD_PORTC;
    SLRCONC = SR_PORTC;
    INLVLC = IL_PORTC;
    //Port E
    PORTE = VL_PORTE;
    WPUE = WP_PORTE;
    INLVLE = IL_PORTE;


    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * Power-down unneeded peripherals to save energy
     *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    PMD0 = CFG_PMD_0;
    PMD1 = CFG_PMD_1;
    PMD2 = CFG_PMD_2;
    PMD3 = CFG_PMD_3;
    PMD4 = CFG_PMD_4;
    PMD5 = CFG_PMD_5;


    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * Configure LCD module
     *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    LCDCON = CFG_LCD_CON;
    LCDPS = CFG_LCD_PS;
    LCDSE0 = CFG_LCD_SEG0;
    LCDSE1 = CFG_LCD_SEG1;
    LCDSE2 = CFG_LCD_SEG2;
    LCDVCON2 = CFG_LCD_VC2;
    LCDDATA0 = CFG_LCD_DAT0;
    LCDDATA1 = CFG_LCD_DAT1;
    LCDDATA2 = CFG_LCD_DAT2;


    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * Configure FVR
     *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    FVRCON = CFG_FVR_CON;


    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * Configure ADC
     *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    ADCON0 = CFG_ADC_CON0;
    ADCON1 = CFG_ADC_CON1;
    ADCON2 = CFG_ADC_CON2;
    ADCON3 = CFG_ADC_CON3;
    ADCLK = CFG_ADC_CLK;
    ADREF = CFG_ADC_REF;
    ADPCH = CFG_ADC_PCH;
    ADPRE = CFG_ADC_PRE;
    ADACQ = CFG_ADC_ACQ;
    ADCAP = CFG_ADC_CAP;
    ADRPT = CFG_ADC_RPT;
    ADACT = CFG_ADC_ACT;


    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * Configure watchdog timer
     *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    WDTCON0 = CFG_WCHD_CON0;
    WDTCON1 = CFG_WCHD_CON1;


    PEIE = ENABLE;                      //Enable peripheral interrupts
    LCDEN = TRUE;                       //Enable LCD module

    return;
} //End function Initialize



//End of file